/* nih-dbus-tool
 *
 * test_output.c - test suite for nih-dbus-tool/output.c
 *
 * Copyright © 2009 Scott James Remnant <scott@netsplit.com>.
 * Copyright © 2009 Canonical Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <nih/test.h>

#include <stdio.h>

#include <nih/macros.h>
#include <nih/alloc.h>
#include <nih/main.h>
#include <nih/error.h>

#include "method.h"
#include "signal.h"
#include "property.h"
#include "node.h"
#include "output.h"


void
test_output (void)
{
	FILE *        source;
	FILE *        header;
	Node *        node = NULL;
	Interface *   interface = NULL;
	Method *      method = NULL;
	Signal *      signal = NULL;
	Argument *    argument = NULL;
	Property *    property = NULL;
	int           ret;
	NihError *    err;

	TEST_FUNCTION ("output");
	source = tmpfile ();
	header = tmpfile ();


	/* Check that we can generate a valid source file and accompanying
	 * header file for a node in proxy mode.
	 */
	TEST_FEATURE ("with proxy");
	TEST_ALLOC_FAIL {
		TEST_ALLOC_SAFE {
			node = node_new (NULL, NULL);

			interface = interface_new (node, "com.netsplit.Nih.Test");
			interface->symbol = "test";
			nih_list_add (&node->interfaces, &interface->entry);

			method = method_new (interface, "Poke");
			method->symbol = "poke";
			nih_list_add (&interface->methods, &method->entry);

			argument = argument_new (method, "address",
						 "u", NIH_DBUS_ARG_IN);
			argument->symbol = "address";
			nih_list_add (&method->arguments, &argument->entry);

			argument = argument_new (method, "value",
						 "s", NIH_DBUS_ARG_IN);
			argument->symbol = "value";
			nih_list_add (&method->arguments, &argument->entry);

			method = method_new (interface, "Peek");
			method->symbol = "peek";
			nih_list_add (&interface->methods, &method->entry);

			argument = argument_new (method, "address",
						 "u", NIH_DBUS_ARG_IN);
			argument->symbol = "address";
			nih_list_add (&method->arguments, &argument->entry);

			argument = argument_new (method, "value",
						 "s", NIH_DBUS_ARG_OUT);
			argument->symbol = "value";
			nih_list_add (&method->arguments, &argument->entry);

			method = method_new (interface, "IsValidAddress");
			method->symbol = "is_valid_address";
			nih_list_add (&interface->methods, &method->entry);

			argument = argument_new (method, "address",
						 "u", NIH_DBUS_ARG_IN);
			argument->symbol = "address";
			nih_list_add (&method->arguments, &argument->entry);


			signal = signal_new (interface, "Bounce");
			signal->symbol = "bounce";
			nih_list_add (&interface->signals, &signal->entry);

			argument = argument_new (signal, "height",
						 "u", NIH_DBUS_ARG_OUT);
			argument->symbol = "height";
			nih_list_add (&signal->arguments, &argument->entry);

			argument = argument_new (signal, "velocity",
						 "i", NIH_DBUS_ARG_OUT);
			argument->symbol = "velocity";
			nih_list_add (&signal->arguments, &argument->entry);

			signal = signal_new (interface, "Exploded");
			signal->symbol = "exploded";
			nih_list_add (&interface->signals, &signal->entry);


			property = property_new (interface, "colour",
						 "s", NIH_DBUS_READWRITE);
			property->symbol = "colour";
			nih_list_add (&interface->properties, &property->entry);

			property = property_new (interface, "size",
						 "u", NIH_DBUS_READ);
			property->symbol = "size";
			nih_list_add (&interface->properties, &property->entry);

			property = property_new (interface, "touch",
						 "b", NIH_DBUS_WRITE);
			property->symbol = "touch";
			nih_list_add (&interface->properties, &property->entry);


			interface = interface_new (node, "com.netsplit.Nih.Foo");
			interface->symbol = "foo";
			nih_list_add (&node->interfaces, &interface->entry);

			method = method_new (interface, "Bing");
			method->symbol = "bing";
			nih_list_add (&interface->methods, &method->entry);

			signal = signal_new (interface, "NewResult");
			signal->symbol = "new_result";
			nih_list_add (&interface->signals, &signal->entry);
		}

		ret = output ("test.c", fileno (source),
			      "test.h", fileno (header),
			      "my", node, FALSE);

		rewind (source);
		rewind (header);

		if (test_alloc_failed) {
			TEST_LT (ret, 0);

			err = nih_error_get ();
			TEST_EQ (err->number, ENOMEM);
			nih_free (err);

			TEST_FILE_RESET (source);
			TEST_FILE_RESET (header);

			nih_free (node);
			continue;
		}

		TEST_EQ (ret, 0);

		TEST_FILE_EQ (source, "/* test\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * test.c - auto-generated D-Bus bindings\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (source, " * conditions.\n");
		TEST_FILE_EQ (source, " */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#ifdef HAVE_CONFIG_H\n");
		TEST_FILE_EQ (source, "# include <config.h>\n");
		TEST_FILE_EQ (source, "#endif /* HAVE_CONFIG_H */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (source, "#include <nih/alloc.h>\n");
		TEST_FILE_EQ (source, "#include <nih/string.h>\n");
		TEST_FILE_EQ (source, "#include <nih/logging.h>\n");
		TEST_FILE_EQ (source, "#include <nih/error.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_error.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_pending_data.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_proxy.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/errors.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include \"test.h\"\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "/* Forward prototypes for static functions */\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Test_Poke_notify           (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Test_Peek_notify           (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Test_IsValidAddress_notify (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult my_com_netsplit_Nih_Test_Bounce_signal         (DBusConnection *connection, DBusMessage *signal, NihDBusProxySignal *proxied);\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult my_com_netsplit_Nih_Test_Exploded_signal       (DBusConnection *connection, DBusMessage *signal, NihDBusProxySignal *proxied);\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Test_colour_get_notify     (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Test_colour_set_notify     (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Test_size_get_notify       (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Test_touch_set_notify      (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static void              my_com_netsplit_Nih_Foo_Bing_notify            (DBusPendingCall *pending_call, NihDBusPendingData *pending_data);\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult my_com_netsplit_Nih_Foo_NewResult_signal       (DBusConnection *connection, DBusMessage *signal, NihDBusProxySignal *proxied);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Poke_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"address\", \"u\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ \"value\",   \"s\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Peek_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"address\", \"u\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ \"value\",   \"s\", NIH_DBUS_ARG_OUT },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_IsValidAddress_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"address\", \"u\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusMethod my_com_netsplit_Nih_Test_methods[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"Poke\",           my_com_netsplit_Nih_Test_Poke_method_args,           NULL },\n");
		TEST_FILE_EQ (source, "\t{ \"Peek\",           my_com_netsplit_Nih_Test_Peek_method_args,           NULL },\n");
		TEST_FILE_EQ (source, "\t{ \"IsValidAddress\", my_com_netsplit_Nih_Test_IsValidAddress_method_args, NULL },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Bounce_signal_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"height\",   \"u\", NIH_DBUS_ARG_OUT },\n");
		TEST_FILE_EQ (source, "\t{ \"velocity\", \"i\", NIH_DBUS_ARG_OUT },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Exploded_signal_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusSignal my_com_netsplit_Nih_Test_signals[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"Bounce\",   my_com_netsplit_Nih_Test_Bounce_signal_args,   my_com_netsplit_Nih_Test_Bounce_signal   },\n");
		TEST_FILE_EQ (source, "\t{ \"Exploded\", my_com_netsplit_Nih_Test_Exploded_signal_args, my_com_netsplit_Nih_Test_Exploded_signal },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusProperty my_com_netsplit_Nih_Test_properties[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"colour\", \"s\", NIH_DBUS_READWRITE, NULL, NULL },\n");
		TEST_FILE_EQ (source, "\t{ \"size\",   \"u\", NIH_DBUS_READ,      NULL, NULL },\n");
		TEST_FILE_EQ (source, "\t{ \"touch\",  \"b\", NIH_DBUS_WRITE,     NULL, NULL },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface my_com_netsplit_Nih_Test = {\n");
		TEST_FILE_EQ (source, "\t\"com.netsplit.Nih.Test\",\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Test_methods,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Test_signals,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Test_properties\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Foo_Bing_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusMethod my_com_netsplit_Nih_Foo_methods[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"Bing\", my_com_netsplit_Nih_Foo_Bing_method_args, NULL },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Foo_NewResult_signal_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusSignal my_com_netsplit_Nih_Foo_signals[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"NewResult\", my_com_netsplit_Nih_Foo_NewResult_signal_args, my_com_netsplit_Nih_Foo_NewResult_signal },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusProperty my_com_netsplit_Nih_Foo_properties[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface my_com_netsplit_Nih_Foo = {\n");
		TEST_FILE_EQ (source, "\t\"com.netsplit.Nih.Foo\",\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Foo_methods,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Foo_signals,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Foo_properties\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface *my_interfaces[] = {\n");
		TEST_FILE_EQ (source, "\t&my_com_netsplit_Nih_Test,\n");
		TEST_FILE_EQ (source, "\t&my_com_netsplit_Nih_Foo,\n");
		TEST_FILE_EQ (source, "\tNULL\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_test_poke (NihDBusProxy *      proxy,\n");
		TEST_FILE_EQ (source, "              uint32_t            address,\n");
		TEST_FILE_EQ (source, "              const char *        value,\n");
		TEST_FILE_EQ (source, "              MyTestPokeReply     handler,\n");
		TEST_FILE_EQ (source, "              NihDBusErrorHandler error_handler,\n");
		TEST_FILE_EQ (source, "              void *              data,\n");
		TEST_FILE_EQ (source, "              int                 timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler == NULL) || (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Test\", \"Poke\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &address)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a char * onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle a fire-and-forget message */\n");
		TEST_FILE_EQ (source, "\tif (! error_handler) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_set_no_reply (method_call, TRUE);\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (proxy->connection, method_call, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\treturn (DBusPendingCall *)TRUE;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Test_Poke_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_Poke_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                      NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Create a message context for the reply, and iterate\n");
		TEST_FILE_EQ (source, "\t\t * over its arguments.\n");
		TEST_FILE_EQ (source, "\t\t */\n");
		TEST_FILE_EQ (source, "\t\tmessage = nih_dbus_message_new (pending_data, pending_data->connection, reply);\n");
		TEST_FILE_EQ (source, "\t\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (pending_data->handler) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t((MyTestPokeReply)pending_data->handler) (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_poke_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                   NihDBusProxy *proxy,\n");
		TEST_FILE_EQ (source, "                   uint32_t      address,\n");
		TEST_FILE_EQ (source, "                   const char *  value)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Test\", \"Poke\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &address)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a char * onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments of the reply */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_test_peek (NihDBusProxy *      proxy,\n");
		TEST_FILE_EQ (source, "              uint32_t            address,\n");
		TEST_FILE_EQ (source, "              MyTestPeekReply     handler,\n");
		TEST_FILE_EQ (source, "              NihDBusErrorHandler error_handler,\n");
		TEST_FILE_EQ (source, "              void *              data,\n");
		TEST_FILE_EQ (source, "              int                 timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler == NULL) || (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Test\", \"Peek\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &address)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle a fire-and-forget message */\n");
		TEST_FILE_EQ (source, "\tif (! error_handler) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_set_no_reply (method_call, TRUE);\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (proxy->connection, method_call, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\treturn (DBusPendingCall *)TRUE;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Test_Peek_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_Peek_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                      NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tchar *          value;\n");
		TEST_FILE_EQ (source, "\tconst char *    value_dbus;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Create a message context for the reply, and iterate\n");
		TEST_FILE_EQ (source, "\t\t * over its arguments.\n");
		TEST_FILE_EQ (source, "\t\t */\n");
		TEST_FILE_EQ (source, "\t\tmessage = nih_dbus_message_new (pending_data, pending_data->connection, reply);\n");
		TEST_FILE_EQ (source, "\t\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Demarshal a char * from the message */\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_get_basic (&iter, &value_dbus);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tvalue = nih_strdup (message, value_dbus);\n");
		TEST_FILE_EQ (source, "\t\tif (! value) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tmessage = NULL;\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (pending_data->handler) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t((MyTestPeekReply)pending_data->handler) (pending_data->data, message, value);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_peek_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                   NihDBusProxy *proxy,\n");
		TEST_FILE_EQ (source, "                   uint32_t      address,\n");
		TEST_FILE_EQ (source, "                   char **       value)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tchar *          value_local;\n");
		TEST_FILE_EQ (source, "\tconst char *    value_local_dbus;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Test\", \"Peek\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &address)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments of the reply */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Demarshal a char * from the message */\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_get_basic (&iter, &value_local_dbus);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tvalue_local = nih_strdup (parent, value_local_dbus);\n");
		TEST_FILE_EQ (source, "\t\tif (! value_local) {\n");
		TEST_FILE_EQ (source, "\t\t\t*value = NULL;\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t*value = value_local;\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! *value);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_free (value_local);\n");
		TEST_FILE_EQ (source, "\t\t*value = NULL;\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_test_is_valid_address (NihDBusProxy *            proxy,\n");
		TEST_FILE_EQ (source, "                          uint32_t                  address,\n");
		TEST_FILE_EQ (source, "                          MyTestIsValidAddressReply handler,\n");
		TEST_FILE_EQ (source, "                          NihDBusErrorHandler       error_handler,\n");
		TEST_FILE_EQ (source, "                          void *                    data,\n");
		TEST_FILE_EQ (source, "                          int                       timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler == NULL) || (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Test\", \"IsValidAddress\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &address)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle a fire-and-forget message */\n");
		TEST_FILE_EQ (source, "\tif (! error_handler) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_set_no_reply (method_call, TRUE);\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (proxy->connection, method_call, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\treturn (DBusPendingCall *)TRUE;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Test_IsValidAddress_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_IsValidAddress_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                                NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Create a message context for the reply, and iterate\n");
		TEST_FILE_EQ (source, "\t\t * over its arguments.\n");
		TEST_FILE_EQ (source, "\t\t */\n");
		TEST_FILE_EQ (source, "\t\tmessage = nih_dbus_message_new (pending_data, pending_data->connection, reply);\n");
		TEST_FILE_EQ (source, "\t\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (pending_data->handler) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t((MyTestIsValidAddressReply)pending_data->handler) (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_is_valid_address_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                               NihDBusProxy *proxy,\n");
		TEST_FILE_EQ (source, "                               uint32_t      address)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Test\", \"IsValidAddress\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &address)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments of the reply */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_Bounce_signal (DBusConnection *    connection,\n");
		TEST_FILE_EQ (source, "                                        DBusMessage *       signal,\n");
		TEST_FILE_EQ (source, "                                        NihDBusProxySignal *proxied)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tuint32_t        height;\n");
		TEST_FILE_EQ (source, "\tint32_t         velocity;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (signal != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxied != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection == proxied->connection);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_is_signal (signal, proxied->interface->name, proxied->signal->name))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_has_path (signal, proxied->path))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (proxied->name)\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_message_has_sender (signal, proxied->name))\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tmessage = nih_dbus_message_new (NULL, connection, signal);\n");
		TEST_FILE_EQ (source, "\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments to the signal and demarshal into arguments\n");
		TEST_FILE_EQ (source, "\t * for our own function call.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a uint32_t from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT32) {\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&iter, &height);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a int32_t from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INT32) {\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&iter, &velocity);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t((MyTestBounceHandler)proxied->handler) (proxied->data, message, height, velocity);\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_Exploded_signal (DBusConnection *    connection,\n");
		TEST_FILE_EQ (source, "                                          DBusMessage *       signal,\n");
		TEST_FILE_EQ (source, "                                          NihDBusProxySignal *proxied)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (signal != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxied != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection == proxied->connection);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_is_signal (signal, proxied->interface->name, proxied->signal->name))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_has_path (signal, proxied->path))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (proxied->name)\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_message_has_sender (signal, proxied->name))\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tmessage = nih_dbus_message_new (NULL, connection, signal);\n");
		TEST_FILE_EQ (source, "\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments to the signal and demarshal into arguments\n");
		TEST_FILE_EQ (source, "\t * for our own function call.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t((MyTestExplodedHandler)proxied->handler) (proxied->data, message);\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_test_get_colour (NihDBusProxy *       proxy,\n");
		TEST_FILE_EQ (source, "                    MyTestGetColourReply handler,\n");
		TEST_FILE_EQ (source, "                    NihDBusErrorHandler  error_handler,\n");
		TEST_FILE_EQ (source, "                    void *               data,\n");
		TEST_FILE_EQ (source, "                    int                  timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\tconst char *        interface;\n");
		TEST_FILE_EQ (source, "\tconst char *        property;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler != NULL) && (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Get\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"colour\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Test_colour_get_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_colour_get_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                            NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tconst char *    value_dbus;\n");
		TEST_FILE_EQ (source, "\tchar *          value;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Create a message context for the reply, and iterate\n");
		TEST_FILE_EQ (source, "\t\t * over and recurse into the arguments.\n");
		TEST_FILE_EQ (source, "\t\t */\n");
		TEST_FILE_EQ (source, "\t\tmessage = nih_dbus_message_new (pending_data, pending_data->connection, reply);\n");
		TEST_FILE_EQ (source, "\t\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_VARIANT) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_recurse (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Demarshal a char * from the message */\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&variter) != DBUS_TYPE_STRING) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_get_basic (&variter, &value_dbus);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tvalue = nih_strdup (message, value_dbus);\n");
		TEST_FILE_EQ (source, "\t\tif (! value) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tmessage = NULL;\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t((MyTestGetColourReply)pending_data->handler) (pending_data->data, message, value);\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_get_colour_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                         NihDBusProxy *proxy,\n");
		TEST_FILE_EQ (source, "                         char **       value)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tconst char *    interface;\n");
		TEST_FILE_EQ (source, "\tconst char *    property;\n");
		TEST_FILE_EQ (source, "\tconst char *    local_dbus;\n");
		TEST_FILE_EQ (source, "\tchar *          local;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Get\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"colour\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the method arguments, recursing into the variant */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_VARIANT) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_recurse (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Demarshal a char * from the message */\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&variter) != DBUS_TYPE_STRING) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_get_basic (&variter, &local_dbus);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tlocal = nih_strdup (parent, local_dbus);\n");
		TEST_FILE_EQ (source, "\t\tif (! local) {\n");
		TEST_FILE_EQ (source, "\t\t\t*value = NULL;\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t*value = local;\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! *value);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_test_set_colour (NihDBusProxy *       proxy,\n");
		TEST_FILE_EQ (source, "                    const char *         value,\n");
		TEST_FILE_EQ (source, "                    MyTestSetColourReply handler,\n");
		TEST_FILE_EQ (source, "                    NihDBusErrorHandler  error_handler,\n");
		TEST_FILE_EQ (source, "                    void *               data,\n");
		TEST_FILE_EQ (source, "                    int                  timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     variter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\tconst char *        interface;\n");
		TEST_FILE_EQ (source, "\tconst char *        property;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler == NULL) || (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Set\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"colour\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, \"s\", &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a char * onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&variter, DBUS_TYPE_STRING, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_abandon_container (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_close_container (&iter, &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle a fire-and-forget message */\n");
		TEST_FILE_EQ (source, "\tif (! error_handler) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_set_no_reply (method_call, TRUE);\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (proxy->connection, method_call, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\treturn (DBusPendingCall *)TRUE;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Test_colour_set_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_colour_set_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                            NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Create a message context for the reply, and check\n");
		TEST_FILE_EQ (source, "\t * there are no arguments.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (pending_data->handler) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t((MyTestSetColourReply)pending_data->handler) (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_set_colour_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                         NihDBusProxy *proxy,\n");
		TEST_FILE_EQ (source, "                         const char *  value)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tconst char *    interface;\n");
		TEST_FILE_EQ (source, "\tconst char *    property;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Set\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"colour\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, \"s\", &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a char * onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&variter, DBUS_TYPE_STRING, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_abandon_container (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_close_container (&iter, &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Check the reply has no arguments */\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_test_get_size (NihDBusProxy *      proxy,\n");
		TEST_FILE_EQ (source, "                  MyTestGetSizeReply  handler,\n");
		TEST_FILE_EQ (source, "                  NihDBusErrorHandler error_handler,\n");
		TEST_FILE_EQ (source, "                  void *              data,\n");
		TEST_FILE_EQ (source, "                  int                 timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\tconst char *        interface;\n");
		TEST_FILE_EQ (source, "\tconst char *        property;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler != NULL) && (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Get\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"size\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Test_size_get_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_size_get_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                          NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tuint32_t        value;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Create a message context for the reply, and iterate\n");
		TEST_FILE_EQ (source, "\t\t * over and recurse into the arguments.\n");
		TEST_FILE_EQ (source, "\t\t */\n");
		TEST_FILE_EQ (source, "\t\tmessage = nih_dbus_message_new (pending_data, pending_data->connection, reply);\n");
		TEST_FILE_EQ (source, "\t\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_VARIANT) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_recurse (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Demarshal a uint32_t from the message */\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&variter) != DBUS_TYPE_UINT32) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_get_basic (&variter, &value);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t((MyTestGetSizeReply)pending_data->handler) (pending_data->data, message, value);\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_get_size_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                       NihDBusProxy *proxy,\n");
		TEST_FILE_EQ (source, "                       uint32_t *    value)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tconst char *    interface;\n");
		TEST_FILE_EQ (source, "\tconst char *    property;\n");
		TEST_FILE_EQ (source, "\tuint32_t        local;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Get\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"size\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the method arguments, recursing into the variant */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_VARIANT) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_recurse (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Demarshal a uint32_t from the message */\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&variter) != DBUS_TYPE_UINT32) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_get_basic (&variter, &local);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_next (&variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t*value = local;\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! *value);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_test_set_touch (NihDBusProxy *      proxy,\n");
		TEST_FILE_EQ (source, "                   int                 value,\n");
		TEST_FILE_EQ (source, "                   MyTestSetTouchReply handler,\n");
		TEST_FILE_EQ (source, "                   NihDBusErrorHandler error_handler,\n");
		TEST_FILE_EQ (source, "                   void *              data,\n");
		TEST_FILE_EQ (source, "                   int                 timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     variter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\tconst char *        interface;\n");
		TEST_FILE_EQ (source, "\tconst char *        property;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler == NULL) || (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Set\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"touch\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, \"b\", &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a int onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&variter, DBUS_TYPE_BOOLEAN, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_abandon_container (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_close_container (&iter, &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle a fire-and-forget message */\n");
		TEST_FILE_EQ (source, "\tif (! error_handler) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_set_no_reply (method_call, TRUE);\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (proxy->connection, method_call, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\treturn (DBusPendingCall *)TRUE;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Test_touch_set_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_touch_set_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                           NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Create a message context for the reply, and check\n");
		TEST_FILE_EQ (source, "\t * there are no arguments.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (pending_data->handler) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t((MyTestSetTouchReply)pending_data->handler) (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_set_touch_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                        NihDBusProxy *proxy,\n");
		TEST_FILE_EQ (source, "                        int           value)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tconst char *    interface;\n");
		TEST_FILE_EQ (source, "\tconst char *    property;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"org.freedesktop.DBus.Properties\", \"Set\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tinterface = \"com.netsplit.Nih.Test\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &interface)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tproperty = \"touch\";\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &property)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, \"b\", &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a int onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&variter, DBUS_TYPE_BOOLEAN, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_abandon_container (&iter, &variter);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_close_container (&iter, &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Check the reply has no arguments */\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "DBusPendingCall *\n");
		TEST_FILE_EQ (source, "my_foo_bing (NihDBusProxy *      proxy,\n");
		TEST_FILE_EQ (source, "             MyFooBingReply      handler,\n");
		TEST_FILE_EQ (source, "             NihDBusErrorHandler error_handler,\n");
		TEST_FILE_EQ (source, "             void *              data,\n");
		TEST_FILE_EQ (source, "             int                 timeout)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *       method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter     iter;\n");
		TEST_FILE_EQ (source, "\tDBusPendingCall *   pending_call;\n");
		TEST_FILE_EQ (source, "\tNihDBusPendingData *pending_data;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert ((handler == NULL) || (error_handler != NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Foo\", \"Bing\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle a fire-and-forget message */\n");
		TEST_FILE_EQ (source, "\tif (! error_handler) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_set_no_reply (method_call, TRUE);\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (proxy->connection, method_call, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\treturn (DBusPendingCall *)TRUE;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message and set up the reply notification. */\n");
		TEST_FILE_EQ (source, "\tpending_data = nih_dbus_pending_data_new (NULL, proxy->connection,\n");
		TEST_FILE_EQ (source, "\t                                          (NihDBusReplyHandler)handler,\n");
		TEST_FILE_EQ (source, "\t                                          error_handler, data);\n");
		TEST_FILE_EQ (source, "\tif (! pending_data) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tpending_call = NULL;\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send_with_reply (proxy->connection, method_call,\n");
		TEST_FILE_EQ (source, "\t                                       &pending_call, timeout)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (pending_data);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (NULL);\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_pending_call_set_notify (pending_call, (DBusPendingCallNotifyFunction)my_com_netsplit_Nih_Foo_Bing_notify,\n");
		TEST_FILE_EQ (source, "\t                                        pending_data, (DBusFreeFunction)nih_discard));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn pending_call;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static void\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Foo_Bing_notify (DBusPendingCall *   pending_call,\n");
		TEST_FILE_EQ (source, "                                     NihDBusPendingData *pending_data)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_call != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (pending_data != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_pending_call_get_completed (pending_call));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Steal the reply from the pending call. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_pending_call_steal_reply (pending_call);\n");
		TEST_FILE_EQ (source, "\tnih_assert (reply != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Handle error replies */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\tmessage = NIH_MUST (nih_dbus_message_new (pending_data, pending_data->connection, reply));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\t\tdbus_set_error_from_message (&error, message->message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_METHOD_RETURN);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Create a message context for the reply, and iterate\n");
		TEST_FILE_EQ (source, "\t\t * over its arguments.\n");
		TEST_FILE_EQ (source, "\t\t */\n");
		TEST_FILE_EQ (source, "\t\tmessage = nih_dbus_message_new (pending_data, pending_data->connection, reply);\n");
		TEST_FILE_EQ (source, "\t\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise (NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t\t                 _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t\t\tpending_data->error_handler (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (pending_data->handler) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t\t((MyFooBingReply)pending_data->handler) (pending_data->data, message);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_foo_bing_sync (const void *  parent,\n");
		TEST_FILE_EQ (source, "                  NihDBusProxy *proxy)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   method_call;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusError       error;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxy != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the method call message. */\n");
		TEST_FILE_EQ (source, "\tmethod_call = dbus_message_new_method_call (proxy->name, proxy->path, \"com.netsplit.Nih.Foo\", \"Bing\");\n");
		TEST_FILE_EQ (source, "\tif (! method_call)\n");
		TEST_FILE_EQ (source, "\t\tnih_return_no_memory_error (-1);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (method_call, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the message, and wait for the reply. */\n");
		TEST_FILE_EQ (source, "\tdbus_error_init (&error);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treply = dbus_connection_send_with_reply_and_block (proxy->connection, method_call, -1, &error);\n");
		TEST_FILE_EQ (source, "\tif (! reply) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_dbus_error_raise (error.name, error.message);\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_error_free (&error);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (method_call);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments of the reply */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\tnih_return_error (-1, NIH_DBUS_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                  _(NIH_DBUS_INVALID_ARGS_STR));\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Foo_NewResult_signal (DBusConnection *    connection,\n");
		TEST_FILE_EQ (source, "                                          DBusMessage *       signal,\n");
		TEST_FILE_EQ (source, "                                          NihDBusProxySignal *proxied)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tNihDBusMessage *message;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (signal != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (proxied != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection == proxied->connection);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_is_signal (signal, proxied->interface->name, proxied->signal->name))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_has_path (signal, proxied->path))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (proxied->name)\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_message_has_sender (signal, proxied->name))\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tmessage = nih_dbus_message_new (NULL, connection, signal);\n");
		TEST_FILE_EQ (source, "\tif (! message)\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments to the signal and demarshal into arguments\n");
		TEST_FILE_EQ (source, "\t * for our own function call.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\t((MyFooNewResultHandler)proxied->handler) (proxied->data, message);\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\tnih_free (message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn DBUS_HANDLER_RESULT_NOT_YET_HANDLED;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_END (source);
		TEST_FILE_RESET (source);


		TEST_FILE_EQ (header, "/* test\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (header, " * conditions.\n");
		TEST_FILE_EQ (header, " */\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#ifndef TEST_TEST_H\n");
		TEST_FILE_EQ (header, "#define TEST_TEST_H\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_interface.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_pending_data.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_proxy.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestPokeReply) (void *data, NihDBusMessage *message);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestPeekReply) (void *data, NihDBusMessage *message, const char *value);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestIsValidAddressReply) (void *data, NihDBusMessage *message);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestBounceHandler) (void *data, NihDBusMessage *message, uint32_t height, int32_t velocity);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestExplodedHandler) (void *data, NihDBusMessage *message);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestGetColourReply) (void *data, NihDBusMessage *message, const char *value);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestSetColourReply) (void *data, NihDBusMessage *message);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestGetSizeReply) (void *data, NihDBusMessage *message, uint32_t value);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyTestSetTouchReply) (void *data, NihDBusMessage *message);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyFooBingReply) (void *data, NihDBusMessage *message);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "typedef void (*MyFooNewResultHandler) (void *data, NihDBusMessage *message);\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_BEGIN_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface  my_com_netsplit_Nih_Test;\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface  my_com_netsplit_Nih_Foo;\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface *my_interfaces[];\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_test_poke                  (NihDBusProxy *proxy, uint32_t address, const char *value, MyTestPokeReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_test_poke_sync             (const void *parent, NihDBusProxy *proxy, uint32_t address, const char *value)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_test_peek                  (NihDBusProxy *proxy, uint32_t address, MyTestPeekReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_test_peek_sync             (const void *parent, NihDBusProxy *proxy, uint32_t address, char **value)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_test_is_valid_address      (NihDBusProxy *proxy, uint32_t address, MyTestIsValidAddressReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_test_is_valid_address_sync (const void *parent, NihDBusProxy *proxy, uint32_t address)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_test_get_colour            (NihDBusProxy *proxy, MyTestGetColourReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_test_get_colour_sync       (const void *parent, NihDBusProxy *proxy, char **value)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_test_set_colour            (NihDBusProxy *proxy, const char *value, MyTestSetColourReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_test_set_colour_sync       (const void *parent, NihDBusProxy *proxy, const char *value)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_test_get_size              (NihDBusProxy *proxy, MyTestGetSizeReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_test_get_size_sync         (const void *parent, NihDBusProxy *proxy, uint32_t *value)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_test_set_touch             (NihDBusProxy *proxy, int value, MyTestSetTouchReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_test_set_touch_sync        (const void *parent, NihDBusProxy *proxy, int value)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "DBusPendingCall *my_foo_bing                   (NihDBusProxy *proxy, MyFooBingReply handler, NihDBusErrorHandler error_handler, void *data, int timeout)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int              my_foo_bing_sync              (const void *parent, NihDBusProxy *proxy)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_END_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#endif /* TEST_TEST_H */\n");
		TEST_FILE_END (header);
		TEST_FILE_RESET (header);

		nih_free (node);
	}


	/* Check that when there are no interfaces, a valid empty source
	 * and header file are generated.
	 */
	TEST_FEATURE ("with proxy but no interfaces");
	TEST_ALLOC_FAIL {
		TEST_ALLOC_SAFE {
			node = node_new (NULL, NULL);
		}

		ret = output ("test.c", fileno (source),
			      "test.h", fileno (header),
			      "my", node, FALSE);

		rewind (source);
		rewind (header);

		if (test_alloc_failed) {
			TEST_LT (ret, 0);

			err = nih_error_get ();
			TEST_EQ (err->number, ENOMEM);
			nih_free (err);

			TEST_FILE_RESET (source);
			TEST_FILE_RESET (header);

			nih_free (node);
			continue;
		}

		TEST_EQ (ret, 0);

		TEST_FILE_EQ (source, "/* test\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * test.c - auto-generated D-Bus bindings\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (source, " * conditions.\n");
		TEST_FILE_EQ (source, " */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#ifdef HAVE_CONFIG_H\n");
		TEST_FILE_EQ (source, "# include <config.h>\n");
		TEST_FILE_EQ (source, "#endif /* HAVE_CONFIG_H */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (source, "#include <nih/alloc.h>\n");
		TEST_FILE_EQ (source, "#include <nih/string.h>\n");
		TEST_FILE_EQ (source, "#include <nih/logging.h>\n");
		TEST_FILE_EQ (source, "#include <nih/error.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_error.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_pending_data.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_proxy.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/errors.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include \"test.h\"\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface *my_interfaces[] = {\n");
		TEST_FILE_EQ (source, "\tNULL\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_END (source);
		TEST_FILE_RESET (source);


		TEST_FILE_EQ (header, "/* test\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (header, " * conditions.\n");
		TEST_FILE_EQ (header, " */\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#ifndef TEST_TEST_H\n");
		TEST_FILE_EQ (header, "#define TEST_TEST_H\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_interface.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_pending_data.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_proxy.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_BEGIN_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface *my_interfaces[];\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_END_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#endif /* TEST_TEST_H */\n");
		TEST_FILE_END (header);
		TEST_FILE_RESET (header);

		nih_free (node);
	}


	/* Check that we can generate a valid source file and accompanying
	 * header file for a node in object mode.
	 */
	TEST_FEATURE ("with object");
	TEST_ALLOC_FAIL {
		TEST_ALLOC_SAFE {
			node = node_new (NULL, NULL);

			interface = interface_new (node, "com.netsplit.Nih.Test");
			interface->symbol = "test";
			nih_list_add (&node->interfaces, &interface->entry);

			method = method_new (interface, "Poke");
			method->symbol = "poke";
			nih_list_add (&interface->methods, &method->entry);

			argument = argument_new (method, "address",
						 "u", NIH_DBUS_ARG_IN);
			argument->symbol = "address";
			nih_list_add (&method->arguments, &argument->entry);

			argument = argument_new (method, "value",
						 "s", NIH_DBUS_ARG_IN);
			argument->symbol = "value";
			nih_list_add (&method->arguments, &argument->entry);

			method = method_new (interface, "Peek");
			method->symbol = "peek";
			method->async = TRUE;
			nih_list_add (&interface->methods, &method->entry);

			argument = argument_new (method, "address",
						 "u", NIH_DBUS_ARG_IN);
			argument->symbol = "address";
			nih_list_add (&method->arguments, &argument->entry);

			argument = argument_new (method, "value",
						 "s", NIH_DBUS_ARG_OUT);
			argument->symbol = "value";
			nih_list_add (&method->arguments, &argument->entry);

			method = method_new (interface, "IsValidAddress");
			method->symbol = "is_valid_address";
			nih_list_add (&interface->methods, &method->entry);

			argument = argument_new (method, "address",
						 "u", NIH_DBUS_ARG_IN);
			argument->symbol = "address";
			nih_list_add (&method->arguments, &argument->entry);

			argument = argument_new (method, "is_valid",
						 "b", NIH_DBUS_ARG_OUT);
			argument->symbol = "is_valid";
			nih_list_add (&method->arguments, &argument->entry);


			signal = signal_new (interface, "Bounce");
			signal->symbol = "bounce";
			nih_list_add (&interface->signals, &signal->entry);

			argument = argument_new (signal, "height",
						 "u", NIH_DBUS_ARG_OUT);
			argument->symbol = "height";
			nih_list_add (&signal->arguments, &argument->entry);

			argument = argument_new (signal, "velocity",
						 "i", NIH_DBUS_ARG_OUT);
			argument->symbol = "velocity";
			nih_list_add (&signal->arguments, &argument->entry);

			signal = signal_new (interface, "Exploded");
			signal->symbol = "exploded";
			nih_list_add (&interface->signals, &signal->entry);


			property = property_new (interface, "colour",
						 "s", NIH_DBUS_READWRITE);
			property->symbol = "colour";
			nih_list_add (&interface->properties, &property->entry);

			property = property_new (interface, "size",
						 "u", NIH_DBUS_READ);
			property->symbol = "size";
			nih_list_add (&interface->properties, &property->entry);

			property = property_new (interface, "touch",
						 "b", NIH_DBUS_WRITE);
			property->symbol = "touch";
			nih_list_add (&interface->properties, &property->entry);


			interface = interface_new (node, "com.netsplit.Nih.Foo");
			interface->symbol = "foo";
			nih_list_add (&node->interfaces, &interface->entry);

			method = method_new (interface, "Bing");
			method->symbol = "bing";
			nih_list_add (&interface->methods, &method->entry);

			signal = signal_new (interface, "NewResult");
			signal->symbol = "new_result";
			nih_list_add (&interface->signals, &signal->entry);
		}

		ret = output ("test.c", fileno (source),
			      "test.h", fileno (header),
			      "my", node, TRUE);

		rewind (source);
		rewind (header);

		if (test_alloc_failed) {
			TEST_LT (ret, 0);

			err = nih_error_get ();
			TEST_EQ (err->number, ENOMEM);
			nih_free (err);

			TEST_FILE_RESET (source);
			TEST_FILE_RESET (header);

			nih_free (node);
			continue;
		}

		TEST_EQ (ret, 0);

		TEST_FILE_EQ (source, "/* test\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * test.c - auto-generated D-Bus bindings\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (source, " * conditions.\n");
		TEST_FILE_EQ (source, " */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#ifdef HAVE_CONFIG_H\n");
		TEST_FILE_EQ (source, "# include <config.h>\n");
		TEST_FILE_EQ (source, "#endif /* HAVE_CONFIG_H */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (source, "#include <nih/alloc.h>\n");
		TEST_FILE_EQ (source, "#include <nih/string.h>\n");
		TEST_FILE_EQ (source, "#include <nih/logging.h>\n");
		TEST_FILE_EQ (source, "#include <nih/error.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_error.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_object.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/errors.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include \"test.h\"\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "/* Forward prototypes for static functions */\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult my_com_netsplit_Nih_Test_Poke_method           (NihDBusObject *object, NihDBusMessage *message);\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult my_com_netsplit_Nih_Test_Peek_method           (NihDBusObject *object, NihDBusMessage *message);\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult my_com_netsplit_Nih_Test_IsValidAddress_method (NihDBusObject *object, NihDBusMessage *message);\n");
		TEST_FILE_EQ (source, "static int               my_com_netsplit_Nih_Test_colour_get            (NihDBusObject *object, NihDBusMessage *message, DBusMessageIter *iter);\n");
		TEST_FILE_EQ (source, "static int               my_com_netsplit_Nih_Test_colour_set            (NihDBusObject *object, NihDBusMessage *message, DBusMessageIter *iter);\n");
		TEST_FILE_EQ (source, "static int               my_com_netsplit_Nih_Test_size_get              (NihDBusObject *object, NihDBusMessage *message, DBusMessageIter *iter);\n");
		TEST_FILE_EQ (source, "static int               my_com_netsplit_Nih_Test_touch_set             (NihDBusObject *object, NihDBusMessage *message, DBusMessageIter *iter);\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult my_com_netsplit_Nih_Foo_Bing_method            (NihDBusObject *object, NihDBusMessage *message);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "/* Prototypes for externally implemented handler functions */\n");
		TEST_FILE_EQ (source, "extern int my_test_poke             (void *data, NihDBusMessage *message, uint32_t address, const char *value)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "extern int my_test_peek             (void *data, NihDBusMessage *message, uint32_t address)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "extern int my_test_is_valid_address (void *data, NihDBusMessage *message, uint32_t address, int *is_valid)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "extern int my_test_get_colour       (void *data, NihDBusMessage *message, char **value)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "extern int my_test_set_colour       (void *data, NihDBusMessage *message, const char *value)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "extern int my_test_get_size         (void *data, NihDBusMessage *message, uint32_t *value)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "extern int my_test_set_touch        (void *data, NihDBusMessage *message, int value)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "extern int my_foo_bing              (void *data, NihDBusMessage *message)\n");
		TEST_FILE_EQ (source, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Poke_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"address\", \"u\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ \"value\",   \"s\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Peek_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"address\", \"u\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ \"value\",   \"s\", NIH_DBUS_ARG_OUT },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_IsValidAddress_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"address\",  \"u\", NIH_DBUS_ARG_IN  },\n");
		TEST_FILE_EQ (source, "\t{ \"is_valid\", \"b\", NIH_DBUS_ARG_OUT },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusMethod my_com_netsplit_Nih_Test_methods[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"Poke\",           my_com_netsplit_Nih_Test_Poke_method_args,           my_com_netsplit_Nih_Test_Poke_method           },\n");
		TEST_FILE_EQ (source, "\t{ \"Peek\",           my_com_netsplit_Nih_Test_Peek_method_args,           my_com_netsplit_Nih_Test_Peek_method           },\n");
		TEST_FILE_EQ (source, "\t{ \"IsValidAddress\", my_com_netsplit_Nih_Test_IsValidAddress_method_args, my_com_netsplit_Nih_Test_IsValidAddress_method },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Bounce_signal_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"height\",   \"u\", NIH_DBUS_ARG_OUT },\n");
		TEST_FILE_EQ (source, "\t{ \"velocity\", \"i\", NIH_DBUS_ARG_OUT },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Test_Exploded_signal_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusSignal my_com_netsplit_Nih_Test_signals[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"Bounce\",   my_com_netsplit_Nih_Test_Bounce_signal_args,   NULL },\n");
		TEST_FILE_EQ (source, "\t{ \"Exploded\", my_com_netsplit_Nih_Test_Exploded_signal_args, NULL },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusProperty my_com_netsplit_Nih_Test_properties[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"colour\", \"s\", NIH_DBUS_READWRITE, my_com_netsplit_Nih_Test_colour_get, my_com_netsplit_Nih_Test_colour_set },\n");
		TEST_FILE_EQ (source, "\t{ \"size\",   \"u\", NIH_DBUS_READ,      my_com_netsplit_Nih_Test_size_get,   NULL                                },\n");
		TEST_FILE_EQ (source, "\t{ \"touch\",  \"b\", NIH_DBUS_WRITE,     NULL,                                my_com_netsplit_Nih_Test_touch_set  },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface my_com_netsplit_Nih_Test = {\n");
		TEST_FILE_EQ (source, "\t\"com.netsplit.Nih.Test\",\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Test_methods,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Test_signals,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Test_properties\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Foo_Bing_method_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusMethod my_com_netsplit_Nih_Foo_methods[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"Bing\", my_com_netsplit_Nih_Foo_Bing_method_args, my_com_netsplit_Nih_Foo_Bing_method },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusArg my_com_netsplit_Nih_Foo_NewResult_signal_args[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusSignal my_com_netsplit_Nih_Foo_signals[] = {\n");
		TEST_FILE_EQ (source, "\t{ \"NewResult\", my_com_netsplit_Nih_Foo_NewResult_signal_args, NULL },\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static const NihDBusProperty my_com_netsplit_Nih_Foo_properties[] = {\n");
		TEST_FILE_EQ (source, "\t{ NULL }\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface my_com_netsplit_Nih_Foo = {\n");
		TEST_FILE_EQ (source, "\t\"com.netsplit.Nih.Foo\",\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Foo_methods,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Foo_signals,\n");
		TEST_FILE_EQ (source, "\tmy_com_netsplit_Nih_Foo_properties\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface *my_interfaces[] = {\n");
		TEST_FILE_EQ (source, "\t&my_com_netsplit_Nih_Test,\n");
		TEST_FILE_EQ (source, "\t&my_com_netsplit_Nih_Foo,\n");
		TEST_FILE_EQ (source, "\tNULL\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_Poke_method (NihDBusObject * object,\n");
		TEST_FILE_EQ (source, "                                      NihDBusMessage *message)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tuint32_t        address;\n");
		TEST_FILE_EQ (source, "\tchar *          value;\n");
		TEST_FILE_EQ (source, "\tconst char *    value_dbus;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments to the message and demarshal into arguments\n");
		TEST_FILE_EQ (source, "\t * for our own function call.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a uint32_t from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT32) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to Poke method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&iter, &address);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a char * from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to Poke method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&iter, &value_dbus);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tvalue = nih_strdup (message, value_dbus);\n");
		TEST_FILE_EQ (source, "\tif (! value) {\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to Poke method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\tif (my_test_poke (object->data, message, address, value) < 0) {\n");
		TEST_FILE_EQ (source, "\t\tNihError *err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\terr = nih_error_get ();\n");
		TEST_FILE_EQ (source, "\t\tif (err->number == ENOMEM) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t} else if (err->number == NIH_DBUS_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\t\tNihDBusError *dbus_err = (NihDBusError *)err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, dbus_err->name, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, DBUS_ERROR_FAILED, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* If the sender doesn't care about a reply, don't bother wasting\n");
		TEST_FILE_EQ (source, "\t * effort constructing and sending one.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_no_reply (message->message))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Construct the reply message. */\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_method_return (message->message);\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init_append (reply, &iter);\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the reply, appending it to the outgoing queue. */\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_Peek_method (NihDBusObject * object,\n");
		TEST_FILE_EQ (source, "                                      NihDBusMessage *message)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tuint32_t        address;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments to the message and demarshal into arguments\n");
		TEST_FILE_EQ (source, "\t * for our own function call.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a uint32_t from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT32) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to Peek method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&iter, &address);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to Peek method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\tif (my_test_peek (object->data, message, address) < 0) {\n");
		TEST_FILE_EQ (source, "\t\tNihError *err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\terr = nih_error_get ();\n");
		TEST_FILE_EQ (source, "\t\tif (err->number == ENOMEM) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t} else if (err->number == NIH_DBUS_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\t\tNihDBusError *dbus_err = (NihDBusError *)err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, dbus_err->name, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, DBUS_ERROR_FAILED, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_peek_reply (NihDBusMessage *message,\n");
		TEST_FILE_EQ (source, "                    const char *    value)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (value != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* If the sender doesn't care about a reply, don't bother wasting\n");
		TEST_FILE_EQ (source, "\t * effort constructing and sending one.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_no_reply (message->message))\n");
		TEST_FILE_EQ (source, "\t\treturn 0;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the reply message. */\n");
		TEST_FILE_EQ (source, "\treply = dbus_message_new_method_return (message->message);\n");
		TEST_FILE_EQ (source, "\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a char * onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the reply, appending it to the outgoing queue. */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_IsValidAddress_method (NihDBusObject * object,\n");
		TEST_FILE_EQ (source, "                                                NihDBusMessage *message)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\tuint32_t        address;\n");
		TEST_FILE_EQ (source, "\tint             is_valid;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments to the message and demarshal into arguments\n");
		TEST_FILE_EQ (source, "\t * for our own function call.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a uint32_t from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT32) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to IsValidAddress method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&iter, &address);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to IsValidAddress method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\tif (my_test_is_valid_address (object->data, message, address, &is_valid) < 0) {\n");
		TEST_FILE_EQ (source, "\t\tNihError *err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\terr = nih_error_get ();\n");
		TEST_FILE_EQ (source, "\t\tif (err->number == ENOMEM) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t} else if (err->number == NIH_DBUS_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\t\tNihDBusError *dbus_err = (NihDBusError *)err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, dbus_err->name, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, DBUS_ERROR_FAILED, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* If the sender doesn't care about a reply, don't bother wasting\n");
		TEST_FILE_EQ (source, "\t * effort constructing and sending one.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_no_reply (message->message))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Construct the reply message. */\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_method_return (message->message);\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init_append (reply, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Marshal a int onto the message */\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &is_valid)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treply = NULL;\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the reply, appending it to the outgoing queue. */\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_emit_bounce (DBusConnection *connection,\n");
		TEST_FILE_EQ (source, "                     const char *    origin_path,\n");
		TEST_FILE_EQ (source, "                     uint32_t        height,\n");
		TEST_FILE_EQ (source, "                     int32_t         velocity)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   signal;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (origin_path != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the message. */\n");
		TEST_FILE_EQ (source, "\tsignal = dbus_message_new_signal (origin_path, \"com.netsplit.Nih.Test\", \"Bounce\");\n");
		TEST_FILE_EQ (source, "\tif (! signal)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (signal, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &height)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a int32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &velocity)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the signal, appending it to the outgoing queue. */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send (connection, signal, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_test_emit_exploded (DBusConnection *connection,\n");
		TEST_FILE_EQ (source, "                       const char *    origin_path)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   signal;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (origin_path != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the message. */\n");
		TEST_FILE_EQ (source, "\tsignal = dbus_message_new_signal (origin_path, \"com.netsplit.Nih.Test\", \"Exploded\");\n");
		TEST_FILE_EQ (source, "\tif (! signal)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (signal, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the signal, appending it to the outgoing queue. */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send (connection, signal, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static int\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_colour_get (NihDBusObject *  object,\n");
		TEST_FILE_EQ (source, "                                     NihDBusMessage * message,\n");
		TEST_FILE_EQ (source, "                                     DBusMessageIter *iter)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tchar *          value;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (iter != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (my_test_get_colour (object->data, message, &value) < 0)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Append a variant onto the message to contain the property value. */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_open_container (iter, DBUS_TYPE_VARIANT, \"s\", &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a char * onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&variter, DBUS_TYPE_STRING, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_abandon_container (iter, &variter);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Finish the variant */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_close_container (iter, &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static int\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_colour_set (NihDBusObject *  object,\n");
		TEST_FILE_EQ (source, "                                     NihDBusMessage * message,\n");
		TEST_FILE_EQ (source, "                                     DBusMessageIter *iter)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tconst char *    value_dbus;\n");
		TEST_FILE_EQ (source, "\tchar *          value;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (iter != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Recurse into the variant */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_VARIANT) {\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                             \"Invalid arguments to colour property\");\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_recurse (iter, &variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a char * from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&variter) != DBUS_TYPE_STRING) {\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                             \"Invalid arguments to colour property\");\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&variter, &value_dbus);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tvalue = nih_strdup (message, value_dbus);\n");
		TEST_FILE_EQ (source, "\tif (! value) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                             \"Invalid arguments to colour property\");\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (my_test_set_colour (object->data, message, value) < 0)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static int\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_size_get (NihDBusObject *  object,\n");
		TEST_FILE_EQ (source, "                                   NihDBusMessage * message,\n");
		TEST_FILE_EQ (source, "                                   DBusMessageIter *iter)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tuint32_t        value;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (iter != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (my_test_get_size (object->data, message, &value) < 0)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Append a variant onto the message to contain the property value. */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_open_container (iter, DBUS_TYPE_VARIANT, \"u\", &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Marshal a uint32_t onto the message */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_append_basic (&variter, DBUS_TYPE_UINT32, &value)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_abandon_container (iter, &variter);\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Finish the variant */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_message_iter_close_container (iter, &variter)) {\n");
		TEST_FILE_EQ (source, "\t\tnih_error_raise_no_memory ();\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static int\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Test_touch_set (NihDBusObject *  object,\n");
		TEST_FILE_EQ (source, "                                    NihDBusMessage * message,\n");
		TEST_FILE_EQ (source, "                                    DBusMessageIter *iter)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter variter;\n");
		TEST_FILE_EQ (source, "\tint             value;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (iter != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Recurse into the variant */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_VARIANT) {\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                             \"Invalid arguments to touch property\");\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_recurse (iter, &variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Demarshal a int from the message */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&variter) != DBUS_TYPE_BOOLEAN) {\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                             \"Invalid arguments to touch property\");\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_get_basic (&variter, &value);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (&variter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_next (iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\tnih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                             \"Invalid arguments to touch property\");\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tif (my_test_set_touch (object->data, message, value) < 0)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "static DBusHandlerResult\n");
		TEST_FILE_EQ (source, "my_com_netsplit_Nih_Foo_Bing_method (NihDBusObject * object,\n");
		TEST_FILE_EQ (source, "                                     NihDBusMessage *message)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   reply;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (object != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (message != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Iterate the arguments to the message and demarshal into arguments\n");
		TEST_FILE_EQ (source, "\t * for our own function call.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init (message->message, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_error (message->message, DBUS_ERROR_INVALID_ARGS,\n");
		TEST_FILE_EQ (source, "\t\t                                \"Invalid arguments to Bing method\");\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tif (! dbus_connection_send (message->connection, reply, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Call the handler function */\n");
		TEST_FILE_EQ (source, "\tnih_error_push_context ();\n");
		TEST_FILE_EQ (source, "\tif (my_foo_bing (object->data, message) < 0) {\n");
		TEST_FILE_EQ (source, "\t\tNihError *err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\terr = nih_error_get ();\n");
		TEST_FILE_EQ (source, "\t\tif (err->number == ENOMEM) {\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_NEED_MEMORY;\n");
		TEST_FILE_EQ (source, "\t\t} else if (err->number == NIH_DBUS_ERROR) {\n");
		TEST_FILE_EQ (source, "\t\t\tNihDBusError *dbus_err = (NihDBusError *)err;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, dbus_err->name, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t} else {\n");
		TEST_FILE_EQ (source, "\t\t\treply = NIH_MUST (dbus_message_new_error (message->message, DBUS_ERROR_FAILED, err->message));\n");
		TEST_FILE_EQ (source, "\t\t\tnih_free (err);\n");
		TEST_FILE_EQ (source, "\t\t\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\t\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\t\t}\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\tnih_error_pop_context ();\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* If the sender doesn't care about a reply, don't bother wasting\n");
		TEST_FILE_EQ (source, "\t * effort constructing and sending one.\n");
		TEST_FILE_EQ (source, "\t */\n");
		TEST_FILE_EQ (source, "\tif (dbus_message_get_no_reply (message->message))\n");
		TEST_FILE_EQ (source, "\t\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdo {\n");
		TEST_FILE_EQ (source, "\t\t__label__ enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\t/* Construct the reply message. */\n");
		TEST_FILE_EQ (source, "\t\treply = dbus_message_new_method_return (message->message);\n");
		TEST_FILE_EQ (source, "\t\tif (! reply)\n");
		TEST_FILE_EQ (source, "\t\t\tgoto enomem;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_iter_init_append (reply, &iter);\n");
		TEST_FILE_EQ (source, "\tenomem: __attribute__ ((unused));\n");
		TEST_FILE_EQ (source, "\t} while (! reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the reply, appending it to the outgoing queue. */\n");
		TEST_FILE_EQ (source, "\tNIH_MUST (dbus_connection_send (message->connection, reply, NULL));\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (reply);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn DBUS_HANDLER_RESULT_HANDLED;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "int\n");
		TEST_FILE_EQ (source, "my_foo_emit_new_result (DBusConnection *connection,\n");
		TEST_FILE_EQ (source, "                        const char *    origin_path)\n");
		TEST_FILE_EQ (source, "{\n");
		TEST_FILE_EQ (source, "\tDBusMessage *   signal;\n");
		TEST_FILE_EQ (source, "\tDBusMessageIter iter;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tnih_assert (connection != NULL);\n");
		TEST_FILE_EQ (source, "\tnih_assert (origin_path != NULL);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Construct the message. */\n");
		TEST_FILE_EQ (source, "\tsignal = dbus_message_new_signal (origin_path, \"com.netsplit.Nih.Foo\", \"NewResult\");\n");
		TEST_FILE_EQ (source, "\tif (! signal)\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_iter_init_append (signal, &iter);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\t/* Send the signal, appending it to the outgoing queue. */\n");
		TEST_FILE_EQ (source, "\tif (! dbus_connection_send (connection, signal, NULL)) {\n");
		TEST_FILE_EQ (source, "\t\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\t\treturn -1;\n");
		TEST_FILE_EQ (source, "\t}\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\tdbus_message_unref (signal);\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\treturn 0;\n");
		TEST_FILE_EQ (source, "}\n");
		TEST_FILE_END (source);
		TEST_FILE_RESET (source);


		TEST_FILE_EQ (header, "/* test\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (header, " * conditions.\n");
		TEST_FILE_EQ (header, " */\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#ifndef TEST_TEST_H\n");
		TEST_FILE_EQ (header, "#define TEST_TEST_H\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_interface.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_BEGIN_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface  my_com_netsplit_Nih_Test;\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface  my_com_netsplit_Nih_Foo;\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface *my_interfaces[];\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "int my_test_peek_reply     (NihDBusMessage *message, const char *value)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int my_test_emit_bounce    (DBusConnection *connection, const char *origin_path, uint32_t height, int32_t velocity)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int my_test_emit_exploded  (DBusConnection *connection, const char *origin_path)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "int my_foo_emit_new_result (DBusConnection *connection, const char *origin_path)\n");
		TEST_FILE_EQ (header, "\t__attribute__ ((warn_unused_result));\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_END_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#endif /* TEST_TEST_H */\n");
		TEST_FILE_END (header);
		TEST_FILE_RESET (header);

		nih_free (node);
	}


	/* Check that when there are no interfaces, a valid empty source
	 * and header file are generated.
	 */
	TEST_FEATURE ("with object but no interfaces");
	TEST_ALLOC_FAIL {
		TEST_ALLOC_SAFE {
			node = node_new (NULL, NULL);
		}

		ret = output ("test.c", fileno (source),
			      "test.h", fileno (header),
			      "my", node, TRUE);

		rewind (source);
		rewind (header);

		if (test_alloc_failed) {
			TEST_LT (ret, 0);

			err = nih_error_get ();
			TEST_EQ (err->number, ENOMEM);
			nih_free (err);

			TEST_FILE_RESET (source);
			TEST_FILE_RESET (header);

			nih_free (node);
			continue;
		}

		TEST_EQ (ret, 0);

		TEST_FILE_EQ (source, "/* test\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * test.c - auto-generated D-Bus bindings\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (source, " *\n");
		TEST_FILE_EQ (source, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (source, " * conditions.\n");
		TEST_FILE_EQ (source, " */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#ifdef HAVE_CONFIG_H\n");
		TEST_FILE_EQ (source, "# include <config.h>\n");
		TEST_FILE_EQ (source, "#endif /* HAVE_CONFIG_H */\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (source, "#include <nih/alloc.h>\n");
		TEST_FILE_EQ (source, "#include <nih/string.h>\n");
		TEST_FILE_EQ (source, "#include <nih/logging.h>\n");
		TEST_FILE_EQ (source, "#include <nih/error.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_error.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/dbus_object.h>\n");
		TEST_FILE_EQ (source, "#include <nih-dbus/errors.h>\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "#include \"test.h\"\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "const NihDBusInterface *my_interfaces[] = {\n");
		TEST_FILE_EQ (source, "\tNULL\n");
		TEST_FILE_EQ (source, "};\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_EQ (source, "\n");
		TEST_FILE_END (source);
		TEST_FILE_RESET (source);


		TEST_FILE_EQ (header, "/* test\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * Copyright (C) 2009 Joe Bloggs.\n");
		TEST_FILE_EQ (header, " *\n");
		TEST_FILE_EQ (header, " * This file was automatically generated; see the source for copying\n");
		TEST_FILE_EQ (header, " * conditions.\n");
		TEST_FILE_EQ (header, " */\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#ifndef TEST_TEST_H\n");
		TEST_FILE_EQ (header, "#define TEST_TEST_H\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <dbus/dbus.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih/macros.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_interface.h>\n");
		TEST_FILE_EQ (header, "#include <nih-dbus/dbus_message.h>\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_BEGIN_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "extern const NihDBusInterface *my_interfaces[];\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "NIH_END_EXTERN\n");
		TEST_FILE_EQ (header, "\n");
		TEST_FILE_EQ (header, "#endif /* TEST_TEST_H */\n");
		TEST_FILE_END (header);
		TEST_FILE_RESET (header);

		nih_free (node);
	}


	fclose (source);
	fclose (header);
}


void
test_preamble (void)
{
	char *str;

	TEST_FUNCTION ("output_preamble");

	/* Check that a preamble for a source file is correctly generated,
	 * with the package name, source file path and copyright all
	 * present.
	 */
	TEST_FEATURE ("with path");
	TEST_ALLOC_FAIL {
		str = output_preamble (NULL, "path.c");

		if (test_alloc_failed) {
			TEST_EQ_P (str, NULL);
			continue;
		}

		TEST_EQ_STR (str, ("/* test\n"
				   " *\n"
				   " * path.c - auto-generated D-Bus bindings\n"
				   " *\n"
				   " * Copyright (C) 2009 Joe Bloggs.\n"
				   " *\n"
				   " * This file was automatically generated; see the source for copying\n"
				   " * conditions.\n"
				   " */\n"
				   "\n"));

		nih_free (str);
	}


	/* Check that a preamble for a header file (NULL path) is correctly
	 * generated with the package name and copyright present.
	 */
	TEST_FEATURE ("with no path");
	TEST_ALLOC_FAIL {
		str = output_preamble (NULL, NULL);

		if (test_alloc_failed) {
			TEST_EQ_P (str, NULL);
			continue;
		}

		TEST_EQ_STR (str, ("/* test\n"
				   " *\n"
				   " * Copyright (C) 2009 Joe Bloggs.\n"
				   " *\n"
				   " * This file was automatically generated; see the source for copying\n"
				   " * conditions.\n"
				   " */\n"
				   "\n"));

		nih_free (str);
	}
}


void
test_sentinel (void)
{
	char *str;

	TEST_FUNCTION ("output_sentinel");

	/* Check that a header file sentinel is correctly generated for a
	 * local path.
	 */
	TEST_FEATURE ("with local path");
	TEST_ALLOC_FAIL {
		str = output_sentinel (NULL, "foo.h");

		if (test_alloc_failed) {
			TEST_EQ_P (str, NULL);
			continue;
		}

		TEST_EQ_STR (str, "TEST_FOO_H");

		nih_free (str);
	}


	/* Check that a header file sentinel is correctly generated for a
	 * sub-directory path.
	 */
	TEST_FEATURE ("with sub-directory path");
	TEST_ALLOC_FAIL {
		str = output_sentinel (NULL, "path/to/foo.h");

		if (test_alloc_failed) {
			TEST_EQ_P (str, NULL);
			continue;
		}

		TEST_EQ_STR (str, "TEST_PATH_TO_FOO_H");

		nih_free (str);
	}


	/* Check that a header file sentinel is generated for an absolute
	 * path; we might want to change the format of this later, but it's
	 * ok for now.
	 */
	TEST_FEATURE ("with absolute path");
	TEST_ALLOC_FAIL {
		str = output_sentinel (NULL, "/path/to/foo.h");

		if (test_alloc_failed) {
			TEST_EQ_P (str, NULL);
			continue;
		}

		TEST_EQ_STR (str, "TEST__PATH_TO_FOO_H");

		nih_free (str);
	}
}


int
main (int   argc,
      char *argv[])
{
	package_name = "test";
	package_copyright = "Copyright (C) 2009 Joe Bloggs.";
	program_name = "test";
	nih_error_init ();

	test_output ();
	test_preamble ();
	test_sentinel ();

	return 0;
}
